import datetime
import os
import re
import subprocess

from django.core.management.base import BaseCommand, CommandError

from ...models import NonstopFile, Track


def get_duration(filename):
    p = subprocess.Popen(['mediainfo', '--Inform=Audio;%Duration%', filename],
                         close_fds=True,
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    try:
        return int(stdout) / 1000
    except ValueError:
        pass

    # fallback on soxi
    p = subprocess.Popen(['soxi', filename], close_fds=True,
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    for line in stdout.splitlines():
        if not line.startswith('Duration'):
             continue
        try:
            hours, minutes, seconds = re.findall('(\d\d):(\d\d):(\d\d)', line)[0]
        except IndexError:
            continue
        return int(hours)*3600 + int(minutes)*60 + int(seconds)
    return None


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
                '--recent', action='store_true', default=False,
                help='Only do recent files')

    def handle(self, verbosity, **kwargs):
        qs = Track.objects.filter(duration__isnull=True)
        if kwargs.get('recent'):
            qs = qs.filter(creation_timestamp__gt=datetime.date.today() - datetime.timedelta(days=10))
        for track in qs:
            try:
                file_path = track.file_path()
            except AttributeError:
                continue
            if file_path is None or not os.path.exists(file_path):
                continue
            duration = get_duration(file_path)
            if duration:
                track.duration = datetime.timedelta(seconds=float(duration))
                track.save()
